@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007-2011 Tristan Heaven <tristanheaven@gmail.com>
+ *   Copyright (C) 2007-2016 Tristan Heaven <tristan@tristanheaven.net>
  *
  *   This file is part of GtkHash.
  *
@@ -14,7 +14,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with GtkHash. If not, see <http://www.gnu.org/licenses/>.
+ *   along with GtkHash. If not, see <https://gnu.org/licenses/gpl-2.0.txt>.
  */
 
 #ifndef GTKHASH_LIST_H
@@ -22,17 +22,19 @@
 
 #include "hash/hash-func.h"
 
+extern struct list_s {
+	unsigned int rows;
+} list;
+
 void list_init(void);
 void list_update(void);
-void list_append_row(const char *uri);
+void list_append_row(const char *uri, const char *check);
 void list_remove_selection(void);
-char *list_get_uri(const unsigned int row);
-GSList *list_get_all_uris(void);
-unsigned int list_count_rows(void);
-void list_set_digest(const char *uri, const enum hash_func_e id,
-	const char *digest);
-char *list_get_digest(const unsigned int row, const enum hash_func_e id);
-char *list_get_selected_digest(const enum hash_func_e id);
+char *list_get_uri(unsigned int row);
+void list_set_digest(unsigned int row, enum hash_func_e id, const char *digest);
+char *list_get_digest(unsigned int row, enum hash_func_e id);
+char *list_get_selected_digest(enum hash_func_e id);
+void list_check_digests(unsigned int row);
 void list_clear_digests(void);
 void list_clear(void);
 
