@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007-2013 Tristan Heaven <tristanheaven@gmail.com>
+ *   Copyright (C) 2007-2016 Tristan Heaven <tristan@tristanheaven.net>
  *
  *   This file is part of GtkHash.
  *
@@ -14,16 +14,16 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with GtkHash. If not, see <http://www.gnu.org/licenses/>.
+ *   along with GtkHash. If not, see <https://gnu.org/licenses/gpl-2.0.txt>.
  */
 
-#ifndef GTKHASH_NAUTILUS_RROPERTIES_HASH_H
+#ifndef GTKHASH_NAUTILUS_PROPERTIES_HASH_H
 #define GTKHASH_NAUTILUS_PROPERTIES_HASH_H
 
 #include "properties.h"
 
-void gtkhash_properties_hash_start(struct page_s *page, const uint8_t *hmac_key,
-	const size_t key_size);
+void gtkhash_properties_hash_start(struct page_s *page,
+	const uint8_t *hmac_key, size_t key_size);
 void gtkhash_properties_hash_stop(struct page_s *page);
 int gtkhash_properties_hash_funcs_supported(struct page_s *page);
 void gtkhash_properties_hash_init(struct page_s *page);
