@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007-2011 Tristan Heaven <tristanheaven@gmail.com>
+ *   Copyright (C) 2007-2016 Tristan Heaven <tristan@tristanheaven.net>
  *
  *   This file is part of GtkHash.
  *
@@ -14,7 +14,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with GtkHash. If not, see <http://www.gnu.org/licenses/>.
+ *   along with GtkHash. If not, see <https://gnu.org/licenses/gpl-2.0.txt>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -23,7 +23,6 @@
 
 #include <stdlib.h>
 #include <stdbool.h>
-#include <string.h>
 #include <gtk/gtk.h>
 
 #include "properties.h"
@@ -38,24 +37,27 @@ enum {
 	COL_DIGEST
 };
 
-static GtkTreeModelFilter *gtkhash_properties_list_get_filter(
+inline static GtkTreeModelFilter *gtkhash_properties_list_get_filter(
 	struct page_s *page)
 {
 	return GTK_TREE_MODEL_FILTER(gtk_tree_view_get_model(page->treeview));
 }
 
-static GtkTreeModel *gtkhash_properties_list_get_model(struct page_s *page)
+inline static GtkTreeModel *gtkhash_properties_list_get_model(
+	struct page_s *page)
 {
 	GtkTreeModelFilter *filter = gtkhash_properties_list_get_filter(page);
 	return gtk_tree_model_filter_get_model(filter);
 }
 
-static GtkListStore *gtkhash_properties_list_get_store(struct page_s *page)
+inline static GtkListStore *gtkhash_properties_list_get_store(
+	struct page_s *page)
 {
 	return GTK_LIST_STORE(gtkhash_properties_list_get_model(page));
 }
 
-void gtkhash_properties_list_update_enabled(struct page_s *page, char *path_str)
+void gtkhash_properties_list_update_enabled(struct page_s *page,
+	char *path_str)
 {
 	GtkTreeModel *model = gtkhash_properties_list_get_model(page);
 	GtkListStore *store = gtkhash_properties_list_get_store(page);
@@ -71,31 +73,59 @@ void gtkhash_properties_list_update_enabled(struct page_s *page, char *path_str)
 
 	if (!enabled) {
 		// Clear digest for disabled func
-		gtkhash_hash_func_clear_digest(&page->hash_file.funcs[id]);
-		const char *digest = gtkhash_hash_func_get_digest(
-			&page->hash_file.funcs[id], DIGEST_FORMAT_HEX_LOWER);
-		gtk_list_store_set(store, &iter, COL_DIGEST, digest, -1);
+		gtk_list_store_set(store, &iter, COL_DIGEST, "", -1);
 	}
 
-	page->hash_file.funcs[id].enabled = enabled;
+	page->funcs[id].enabled = enabled;
 }
 
-void gtkhash_properties_list_update_digests(struct page_s *page)
+void gtkhash_properties_list_update_hash_func_names(struct page_s *page)
 {
 	GtkTreeModel *model = gtkhash_properties_list_get_model(page);
-	GtkListStore *store = gtkhash_properties_list_get_store(page);
 	GtkTreeIter iter;
 
 	if (!gtk_tree_model_get_iter_first(model, &iter))
 		return;
 
+	GtkListStore *store = gtkhash_properties_list_get_store(page);
+	const bool hmac = gtk_toggle_button_get_active(page->togglebutton_hmac);
+
 	do {
 		int id;
 		gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
 
-		const char *digest = gtkhash_hash_func_get_digest(
-			&page->hash_file.funcs[id], DIGEST_FORMAT_HEX_LOWER);
-		gtk_list_store_set(store, &iter, COL_DIGEST, digest, -1);
+		if (!page->funcs[id].hmac_supported)
+			continue;
+
+		if (hmac) {
+			char *name = g_strdup_printf("HMAC-%s",
+				page->funcs[id].name);
+			gtk_list_store_set(store, &iter, COL_HASH_FUNC, name, -1);
+			g_free(name);
+		} else {
+			gtk_list_store_set(store, &iter, COL_HASH_FUNC,
+				page->funcs[id].name, -1);
+		}
+
+		// Digest is invalid now, so clear it
+		gtk_list_store_set(store, &iter, COL_DIGEST, "", -1);
+	} while (gtk_tree_model_iter_next(model, &iter));
+
+	gtk_tree_view_columns_autosize(page->treeview);
+}
+
+void gtkhash_properties_list_clear_digests(struct page_s *page)
+{
+	GtkTreeModel *model = gtkhash_properties_list_get_model(page);
+	GtkTreeIter iter;
+
+	if (!gtk_tree_model_get_iter_first(model, &iter))
+		return;
+
+	GtkListStore *store = gtkhash_properties_list_get_store(page);
+
+	do {
+		gtk_list_store_set(store, &iter, COL_DIGEST, "", -1);
 	} while (gtk_tree_model_iter_next(model, &iter));
 
 	gtk_tree_view_columns_autosize(page->treeview);
@@ -103,25 +133,52 @@ void gtkhash_properties_list_update_digests(struct page_s *page)
 
 void gtkhash_properties_list_check_digests(struct page_s *page)
 {
-	const char *str_in = gtk_entry_get_text(page->entry_check);
+	const char *check = gtk_entry_get_text(page->entry_check);
 	const char *icon = NULL;
 
-	if (*str_in) {
-		for (int i = 0; i < HASH_FUNCS_N; i++) {
-			if (!page->hash_file.funcs[i].enabled)
-				continue;
+	GtkTreeModel *model = gtkhash_properties_list_get_model(page);
+	GtkTreeIter iter;
 
-			const char *str_out = gtkhash_hash_func_get_digest(
-				&page->hash_file.funcs[i], DIGEST_FORMAT_HEX_LOWER);
-			if (strcasecmp(str_in, str_out) == 0) {
-				icon = GTK_STOCK_YES;
+	if (*check && gtk_tree_model_get_iter_first(model, &iter)) {
+		do {
+			char *digest = NULL;;
+			gtk_tree_model_get(model, &iter, COL_DIGEST, &digest, -1);
+
+			if (g_ascii_strcasecmp(check, digest) == 0) {
+				// FIXME: find a real alternative for GTK_STOCK_YES
+				icon = "gtk-yes";
 				break;
 			}
-		}
+
+			g_free(digest);
+		} while (gtk_tree_model_iter_next(model, &iter));
 	}
 
-	gtk_entry_set_icon_from_stock(page->entry_check, GTK_ENTRY_ICON_SECONDARY,
-		icon);
+	gtk_entry_set_icon_from_icon_name(page->entry_check,
+		GTK_ENTRY_ICON_SECONDARY, icon);
+}
+
+void gtkhash_properties_list_set_digest(struct page_s *page,
+	const enum hash_func_e id, const char *digest)
+{
+	GtkTreeModel *model = gtkhash_properties_list_get_model(page);
+	GtkListStore *store = gtkhash_properties_list_get_store(page);
+	GtkTreeIter iter;
+
+	if (!gtk_tree_model_get_iter_first(model, &iter))
+		g_assert_not_reached();
+
+	do {
+		int row_id;
+		gtk_tree_model_get(model, &iter, COL_ID, &row_id, -1);
+
+		if (row_id == id) {
+			gtk_list_store_set(store, &iter, COL_DIGEST, digest, -1);
+			return;
+		}
+	} while (gtk_tree_model_iter_next(model, &iter));
+
+	g_assert_not_reached();
 }
 
 char *gtkhash_properties_list_get_selected_digest(struct page_s *page)
@@ -171,15 +228,14 @@ void gtkhash_properties_list_init(struct page_s *page)
 	GtkListStore *store = gtkhash_properties_list_get_store(page);
 
 	for (int i = 0; i < HASH_FUNCS_N; i++) {
-		if (!page->hash_file.funcs[i].supported)
+		if (!page->funcs[i].supported)
 			continue;
-		const char *digest = gtkhash_hash_func_get_digest(
-			&page->hash_file.funcs[i], DIGEST_FORMAT_HEX_LOWER);
+
 		gtk_list_store_insert_with_values(store, NULL, i,
 			COL_ID, i,
-			COL_ENABLED, page->hash_file.funcs[i].enabled,
-			COL_HASH_FUNC, page->hash_file.funcs[i].name,
-			COL_DIGEST, digest,
+			COL_ENABLED, page->funcs[i].enabled,
+			COL_HASH_FUNC, page->funcs[i].name,
+			COL_DIGEST, "",
 			-1);
 	}
 
