@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007-2016 Tristan Heaven <tristan@tristanheaven.net>
+ *   Copyright (C) 2007-2017 Tristan Heaven <tristan@tristanheaven.net>
  *
  *   This file is part of GtkHash.
  *
@@ -43,7 +43,6 @@
 #include "properties-list.h"
 #include "properties-hash.h"
 #include "properties-prefs.h"
-#include "properties-resources.h"
 #include "../hash/hash-func.h"
 #include "../hash/hash-file.h"
 
@@ -134,18 +133,31 @@ static void gtkhash_properties_on_cell_toggled(struct page_s *page,
 
 static void gtkhash_properties_on_treeview_popup_menu(struct page_s *page)
 {
+	/* Note: Shift+F10 can trigger this, so it's possible for the pointer
+	   to be outside the window */
+
+#if GTK_CHECK_VERSION(3,22,0)
+	gtk_menu_popup_at_pointer(page->menu, NULL);
+#else
 	gtk_menu_popup(page->menu, NULL, NULL, NULL, NULL, 0,
 		gtk_get_current_event_time());
+#endif
 }
 
 static bool gtkhash_properties_on_treeview_button_press_event(
 	struct page_s *page, GdkEventButton *event)
 {
-	// Right click
-	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
-		gtk_menu_popup(page->menu, NULL, NULL, NULL, NULL, 3,
+#if (GTK_MAJOR_VERSION > 2)
+	if (gdk_event_triggers_context_menu((GdkEvent *)event))
+#else
+	if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3))
+#endif
+#if GTK_CHECK_VERSION(3,22,0)
+		gtk_menu_popup_at_pointer(page->menu, (GdkEvent *)event);
+#else
+		gtk_menu_popup(page->menu, NULL, NULL, NULL, NULL, event->button,
 			gdk_event_get_time((GdkEvent *)event));
-	}
+#endif
 
 	return false;
 }
@@ -363,7 +375,6 @@ static GtkBuilder *gtkhash_properties_init_builder(void)
 	if (G_UNLIKELY(error)) {
 		g_warning("%s", error->message);
 		g_error_free(error);
-
 		return NULL;
 	}
 
@@ -546,7 +557,6 @@ PUBLIC void thunar_extension_initialize(GTypeModule *module);
 PUBLIC void thunar_extension_initialize(GTypeModule *module)
 #endif
 {
-	gtkhash_properties_resources_register_resource();
 	gtkhash_properties_register_type(module);
 
 #if ENABLE_NLS
